@@ -1,4 +1,3 @@
-# $Id: PropertyList.pm 2492 2008-01-16 11:26:16Z comdog $
 package Mac::PropertyList;
 use strict;
 
@@ -11,19 +10,20 @@ use Data::Dumper;
 
 use base qw(Exporter);
 
-@EXPORT_OK = qw( 
-	parse_plist 
+@EXPORT_OK = qw(
+	parse_plist
 	parse_plist_fh
 	parse_plist_file
-	plist_as_string 
+	plist_as_string
 	create_from_hash
-	create_from_array );
+	create_from_array 
+	);
 
 %EXPORT_TAGS = (
 	'all' => \@EXPORT_OK,
 	);
-	
-$VERSION = sprintf "%d.%02d", qw( 1 31 );
+
+$VERSION = '1.32';
 
 =head1 NAME
 
@@ -49,8 +49,8 @@ Mac::PropertyList - work with Mac plists at a low level
 	my $plist = create_from_array( \@array );
 
 	my $plist = Mac::PropertyList::dict->new( \%hash );
-		
-	
+
+
 =head1 DESCRIPTION
 
 This module is a low-level interface to the Mac OS X Property List
@@ -85,7 +85,7 @@ If the parse_plist encounters an empty key tag in a dict structure
 =head2 The Mac::PropertyList classes
 
 A plist can have one or more of any of the plist objects, and we have
-to remember the type of thing so we can go back to the XML format. 
+to remember the type of thing so we can go back to the XML format.
 Perl treats numbers and strings the same, but the plist format
 doesn't.
 
@@ -93,7 +93,7 @@ Therefore, everything Mac::PropertyList creates is an object of some
 sort.  Container objects like Mac::PropertyList::array and
 Mac::PropertyList::dict hold other objects.
 
-There are several types of objects: 
+There are several types of objects:
 
 	Mac::PropertyList::string
 	Mac::PropertyList::data
@@ -161,7 +161,7 @@ will be imported unless you ask for it.
 	use Mac::PropertyList qw( parse_plist );
 
 	use Mac::PropertyList qw( :all );
-	
+
 =over 4
 
 =item parse_plist( TEXT )
@@ -177,14 +177,23 @@ sub parse_plist
 	{
 	my $text = shift;
 
-	# we can handle either 0.9 or 1.0
-	$text =~ s|^<\?xml.*?>\s*<!DOC.*>\s*<plist.*?>\s*||;
-	$text =~ s|\s*</plist>\s*$||;
+	my $plist = do {
+		if( $text =~ /\A<\?xml/ ) # XML plists
+			{
+			# we can handle either 0.9 or 1.0
+			$text =~ s|^<\?xml.*?>\s*<!DOC.*>\s*<plist.*?>\s*||;
+			$text =~ s|\s*</plist>\s*$||;
 
-	my $text_source = new Mac::PropertyList::TextSource( $text );
-	my $plist = read_next( $text_source );
-
-	return $plist;
+			my $text_source = Mac::PropertyList::TextSource->new( $text );
+			read_next( $text_source );
+			}
+		elsif( $text =~ /\Abplist/ ) # binary plist
+			{
+			require Mac::PropertyList::ReadBinary;
+			my $parser = Mac::PropertyList::ReadBinary->new( \$text );
+			$parser->plist;
+			}
+		};
 	}
 
 =item parse_plist_fh( FILEHANDLE )
@@ -192,12 +201,12 @@ sub parse_plist
 Parse the XML plist from FILEHANDLE and return the Mac::PropertyList
 data structure. Returns false if the arguments is not a reference.
 
-You can do this in a couple of ways. You can open the file with a 
+You can do this in a couple of ways. You can open the file with a
 lexical filehandle (since Perl 5.6).
 
 	open my( $fh ), $file or die "...";
 	parse_plist_fh( $fh );
-	
+
 Or, you can use a bareword filehandle and pass a reference to its
 typeglob. I don't recommmend this unless you are using an older
 Perl.
@@ -210,16 +219,16 @@ Perl.
 sub parse_plist_fh
 	{
 	my $fh = shift;
-	
+
 	my $text = do { local $/; <$fh> };
-	
+
 	parse_plist( $text );
 	}
 
 =item parse_plist_file( FILE_PATH )
 
 Parse the XML plist in FILE_PATH and return the Mac::PropertyList
-data structure. Returns false if the file does not exist. 
+data structure. Returns false if the file does not exist.
 
 Alternately, you can pass a filehandle reference, but that just
 calls C<parse_plist_fh> for you.
@@ -231,15 +240,15 @@ sub parse_plist_file
 	my $file = shift;
 
 	if( ref $file ) { return parse_plist_fh( $file ) }
-	
+
 	unless( -e $file )
 		{
 		carp( "parse_plist_file: file [$file] does not exist!" );
 		return;
 		}
-		
+
 	my $text = do { local $/; open my($fh), $file; <$fh> };
-	
+
 	parse_plist( $text );
 	}
 
@@ -247,7 +256,7 @@ sub parse_plist_file
 
 Create a plist dictionary from the hash reference.
 
-The values of the hash can only be simple scalars---not references. 
+The values of the hash can only be simple scalars---not references.
 Reference values are silently ignored.
 
 Returns a string representing the hash in the plist format.
@@ -265,10 +274,10 @@ sub create_from_hash
 	foreach my $key ( keys %$hash )
 		{
 		next if ref $hash->{$key};
-		
+
 		my $bit   = Mac::PropertyList::dict->write_key( $key ) . "\n";
 		my $value = Mac::PropertyList::string->new( $hash->{$key} );
-		
+
 		$bit  .= $value->write . "\n";
 
 		$bit =~ s/^/\t/gm;
@@ -285,7 +294,7 @@ sub create_from_hash
 
 Create a plist array from the array reference.
 
-The values of the array can only be simple scalars---not references. 
+The values of the array can only be simple scalars---not references.
 Reference values are silently ignored.
 
 Returns a string representing the array in the plist format.
@@ -301,9 +310,9 @@ sub create_from_array
 	my $string = "$XML_head" . Mac::PropertyList::array->write_open . "\n";
 
 	foreach my $element ( @$array )
-		{		
+		{
 		my $value = Mac::PropertyList::string->new( $element );
-		
+
 		my $bit  .= $value->write . "\n";
 		$bit =~ s/^/\t/gm;
 
@@ -329,37 +338,38 @@ sub read_next
 	local $_ = '';
 	my $value;
 
-	while (not defined $value) {
-	    croak "Couldn't read anything!" if $source->eof;
-	    $_ .= $source->get_line;
+	while( not defined $value )
+		{
+		croak "Couldn't read anything!" if $source->eof;
+		$_ .= $source->get_line;
 
-	    if( s[^\s* < (string|date|real|integer|data) >
+		if( s[^\s* < (string|date|real|integer|data) >
 			   \s*(.*?)\s* </\1> ][]sx )
-	    {
-		print STDERR "read_next: Found value type [$1] with value [$2]\n"
-		    if $Debug > 1;
-		$value = $Readers{$1}->( $2 );
-	    }
+			{
+			print STDERR "read_next: Found value type [$1] with value [$2]\n"
+				if $Debug > 1;
+			$value = $Readers{$1}->( $2 );
+			}
 	    elsif( s[^\s* < (dict|array) > ][]x )
-	    {
-		print STDERR "\$1 is [$1]\n" if $Debug > 1;
-		$value = $Readers{$1}->( $source );
-	    }
+			{
+			print STDERR "\$1 is [$1]\n" if $Debug > 1;
+			$value = $Readers{$1}->( $source );
+			}
 	    # these next two are some wierd cases i found in the iPhoto Prefs
-	    elsif( s[^\s* < dict / > ][]x )
-	    {
-		$value = Mac::PropertyList::dict->new();
-	    }
+		elsif( s[^\s* < dict / > ][]x )
+			{
+			$value = Mac::PropertyList::dict->new();
+			}
 	    elsif( s[^\s* < array / > ][]x )
-	    {
-		$value = Mac::PropertyList::array->new();
-	    }
+			{
+			$value = Mac::PropertyList::array->new();
+			}
 	    elsif( s[^\s* < (true|false) /> ][]x )
-	    {
-		print STDERR "\$1 is [$1]\n" if $Debug > 1;
-		$value = $Readers{$1}->();
-	    }
-	}
+			{
+			print STDERR "\$1 is [$1]\n" if $Debug > 1;
+			$value = $Readers{$1}->();
+			}
+		}
 	$source->put_line($_);
 	return $value;
 	}
@@ -371,25 +381,31 @@ sub read_dict
 
 	my %hash;
 	local $_ = $source->get_line;
-	while( not s|^\s*</dict>|| ) {
-	    my $key;
-	    while (not defined $key) {
-		if (s[^\s*<key>(.*?)</key>][]s) {
-		    $key = $1;
-#                   Bring this back if you want this behavior:
-#		    croak "Key is empty string!" if $key eq '';
+	while( not s|^\s*</dict>|| )
+		{
+		my $key;
+		while (not defined $key)
+			{
+			if (s[^\s*<key>(.*?)</key>][]s)
+				{
+				$key = $1;
+				# Bring this back if you want this behavior:
+				# croak "Key is empty string!" if $key eq '';
+				}
+			else
+				{
+				croak "Could not read key!" if $source->eof;
+				$_ .= $source->get_line;
+				}
+			}
+
+		print STDERR "read_dict: key is [$1]\n" if $Debug > 1;
+
+		$source->put_line( $_ );
+		$hash{ $key } = read_next( $source );
+		$_ = $source->get_line;
 		}
-		else {
-		    croak "Could not read key!" if $source->eof;
-		    $_ .= $source->get_line;
-		}
-	    }
-	    print STDERR "read_dict: key is [$1]\n" if $Debug > 1;
 
-	    $source->put_line( $_ );
-	    $hash{ $key } = read_next( $source );
-	    $_ = $source->get_line;
-	}
 	$source->put_line( $_ );
 	return Mac::PropertyList::dict->new( \%hash );
 	}
@@ -402,11 +418,13 @@ sub read_array
 	my @array = ();
 
 	local $_ = $source->get_line;
-	while( not s|^\s*</array>|| ) {
-	    $source->put_line( $_ );
-	    push @array, read_next( $source );
-	    $_ = $source->get_line;
-	}
+	while( not s|^\s*</array>|| )
+		{
+		$source->put_line( $_ );
+		push @array, read_next( $source );
+		$_ = $source->get_line;
+		}
+
 	$source->put_line( $_ );
 	return Mac::PropertyList::array->new( \@array );
 	}
@@ -424,8 +442,7 @@ sub read_data
 
 =item plist_as_string
 
-Turn the plist data structure into the plist string, which
-is the form the Mac likes to see it in.
+Return the plist data structure as XML in the Mac Property List format.
 
 =cut
 
@@ -442,35 +459,57 @@ sub plist_as_string
 	return $string;
 	}
 
-# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
+=item plist_as_perl
+
+Return the plist data structure as an unblessed Perl data
+structure. There won't be any C<Mac::PropertyList> objects
+in the results.
+
+=cut
+
+sub plist_as_perl
+	{
+	my $object = CORE::shift;
+
+	my $string = '$VAR = ';
+
+	$string .= $object->as_perl;
+
+	return $string;
+	}
+
+# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
 package Mac::PropertyList::Source;
-sub new { 
-        print STDERR "Creating new $_[0]\n" if $Debug > 1;
-        my $self = bless { buffer => [], source => $_[1] }, $_[0]; 
+sub new
+	{
+	print STDERR "Creating new $_[0]\n" if $Debug > 1;
+	my $self = bless { buffer => [], source => $_[1] }, $_[0];
 	return $self;
 	}
 
 sub eof { (not @{$_[0]->{buffer}}) and $_[0]->source_eof }
 
-sub get_line 
-        {
-        my $self = CORE::shift;
+sub get_line
+	{
+	my $self = CORE::shift;
 
 	local $_ = '';
-	while (defined $_ && /^[\r\n\s]*$/) {
-	    if( @{$self->{buffer}} ) {
-		$_ = shift @{$self->{buffer}};
-	    }
-	    else {
-		$_ = $self->get_source_line;
-	    }
-	}
+	while (defined $_ && /^[\r\n\s]*$/)
+		{
+		if( @{$self->{buffer}} ) {
+			$_ = shift @{$self->{buffer}};
+			}
+			else {
+			$_ = $self->get_source_line;
+			}
+		}
+
 	return $_;
-        }
+	}
 
-sub put_line { unshift @{$_[0]->{buffer}}, $_[1] }  
+sub put_line { unshift @{$_[0]->{buffer}}, $_[1] }
 
-# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
+# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
 package Mac::PropertyList::LineListSource;
 use base qw(Mac::PropertyList::Source);
 
@@ -478,25 +517,25 @@ sub get_source_line { return shift @{$_->{source}} if @{$_->{source}} }
 
 sub source_eof { not @{$_[0]->{source}} }
 
-# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
+# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
 package Mac::PropertyList::TextSource;
 use base qw(Mac::PropertyList::Source);
 
 sub get_source_line
-        {
-	    my $self = CORE::shift;
-	    $self->{source} =~ s/(.*(\r|\n|$))//;
-	    $1;
-        }
+	{
+	my $self = CORE::shift;
+	$self->{source} =~ s/(.*(\r|\n|$))//;
+	$1;
+	}
 
 sub source_eof { not $_[0]->{source} }
 
-# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
+# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
 package Mac::PropertyList::Item;
 sub type_value { ( $_[0]->type, $_[0]->value ) }
 
-sub value      
-	{ 
+sub value
+	{
 	my $ref = $_[0]->type;
 
 	do {
@@ -505,31 +544,31 @@ sub value
 		else                     { ${ $_[0] } }
 		};
 	}
-	
+
 sub type { my $r = ref $_[0] ? ref $_[0] : $_[0]; $r =~ s/.*:://; $r; }
 
-sub new 
-	{ 
-	#print STDERR "Got [@_]\n"; 
+sub new
+	{
+	#print STDERR "Got [@_]\n";
 
-	bless $_[1], $_[0] 
+	bless $_[1], $_[0]
 	}
-	
+
 sub write_open  { $_[0]->write_either(); }
 sub write_close { $_[0]->write_either('/'); }
 
 sub write_either
 	{
 	my $slash = defined $_[1] ? '/' : '';
-	
+
 	my $type = $_[0]->type;
-	
+
 	"<$slash$type>";
 	}
-	
+
 sub write_empty { my $type = $_[0]->type; "<$type/>"; }
-	
-# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
+
+# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
 package Mac::PropertyList::Container;
 use base qw(Mac::PropertyList::Item);
 
@@ -537,21 +576,21 @@ sub new
 	{
 	my $class = CORE::shift;
 	my $item  = CORE::shift;
-	
+
 	if( ref $item )
 		{
 		return bless $item, $class;
 		}
-		
+
 	my $empty = do {
 		   if( $class =~ m/array$/ ) { [] }
 		elsif( $class =~ m/dict$/  ) { {} }
 		};
-		
+
 	$class->SUPER::new( $empty );
 	}
-	
-# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
+
+# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
 package Mac::PropertyList::array;
 use base qw(Mac::PropertyList::Container);
 
@@ -561,11 +600,12 @@ sub pop     { CORE::pop @{ $_[0]->value }   }
 sub push    { }
 sub splice  { }
 sub count   { return scalar @{ $_[0]->value } }
-
-sub values 
-	{ 
-	my @v = map { $_->value } @{ $_[0]->value }; 
-	wantarray ? @v : \@v}
+sub _elements { @{ $_[0]->value } } # the raw, unprocessed elements
+sub values
+	{
+	my @v = map { $_->value } $_[0]->_elements;
+	wantarray ? @v : \@v
+	}
 
 sub as_basic_data
 	{
@@ -587,7 +627,7 @@ sub write
 	foreach my $element ( @$self )
 		{
 		my $bit = $element->write;
-		
+
 		$bit =~ s/^/\t/gm;
 
 		$string .= $bit . "\n";
@@ -598,13 +638,22 @@ sub write
 	return $string;
 	}
 
-# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
+sub as_perl
+	{
+	my $self  = CORE::shift;
+	
+	my @array = map { $_->as_perl } $self->_elements;
+	
+	return \@array;
+	}
+	
+# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
 package Mac::PropertyList::dict;
 use base qw(Mac::PropertyList::Container);
 
 sub new {
 	#print STDERR Data::Dumper::Dumper( $_[1] );
-	
+
 	$_[0]->SUPER::new( $_[1] );
 	}
 
@@ -616,26 +665,26 @@ sub value
 	{
 	my $self = shift;
 	my $key  = shift;
-	
+
 	do
 		{
 		if( defined $key )
 			{
 			my $hash = $self->SUPER::value;
-			
+
 			if( exists $hash->{$key} ) { $hash->{$key}->value }
 			else                       { return }
 			}
 		else { $self->SUPER::value }
 		};
-			
+
 	}
-	
+
 sub keys   { my @k = CORE::keys %{ $_[0]->value }; wantarray ? @k : \@k; }
-sub values 
-	{ 
-	my @v = map { $_->value } CORE::values %{ $_[0]->value }; 
-	wantarray ? @v : \@v; 
+sub values
+	{
+	my @v = map { $_->value } CORE::values %{ $_[0]->value };
+	wantarray ? @v : \@v;
 	}
 
 sub as_basic_data
@@ -647,7 +696,7 @@ sub as_basic_data
 		my ($k, $v) = ($_, $self->{$_});
 		$k => eval { $v->can('as_basic_data') } ? $v->as_basic_data : $v
 		} CORE::keys %$self;
-	
+
 	return \%dict;
 	}
 
@@ -676,7 +725,21 @@ sub write
 	return $string;
 	}
 
-# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
+sub as_perl
+	{
+	my $self  = CORE::shift;
+	
+	my %dict = map { 
+		my $v = $self->value($_);
+		$v = $v->as_perl if eval { $v->can( 'as_perl' ) };
+		$_, $v 
+		} $self->keys;
+	
+	return \%dict;
+	}
+
+
+# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
 package Mac::PropertyList::Scalar;
 use base qw(Mac::PropertyList::Item);
 
@@ -685,24 +748,32 @@ sub new { my $copy = $_[1]; $_[0]->SUPER::new( \$copy ) }
 sub as_basic_data { $_[0]->value }
 
 sub write { $_[0]->write_open . $_[0]->value . $_[0]->write_close }
-	
-# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
+
+sub as_perl { $_[0]->value }
+
+# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
 package Mac::PropertyList::date;
 use base qw(Mac::PropertyList::Scalar);
 
-# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
+# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
 package Mac::PropertyList::real;
 use base qw(Mac::PropertyList::Scalar);
 
-# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
+# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
 package Mac::PropertyList::integer;
 use base qw(Mac::PropertyList::Scalar);
 
-# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
+# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
 package Mac::PropertyList::string;
 use base qw(Mac::PropertyList::Scalar);
 
-# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
+# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
+package Mac::PropertyList::ustring;
+use base qw(Mac::PropertyList::Scalar);
+
+# XXX need to do some fancy unicode checking here
+
+# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
 package Mac::PropertyList::data;
 use base qw(Mac::PropertyList::Scalar);
 
@@ -717,45 +788,42 @@ sub write
 
 	my $string = MIME::Base64::encode_base64($value);
 
-	$self->write_open . $string . $self->write_close; 
+	$self->write_open . $string . $self->write_close;
 	}
 
-# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
+# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
 package Mac::PropertyList::Boolean;
 use base qw(Mac::PropertyList::Item);
 
 sub new
 	{
 	my $class = shift;
-	
+
 	my( $type ) = $class =~ m/.*::(.*)/g;
-		
+
 	$class->either( $type );
 	}
-	
+
 sub either { my $copy = $_[1]; bless \$copy, $_[0]  }
 
 sub write  { $_[0]->write_empty }
 
-# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
+# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
 package Mac::PropertyList::true;
 use base qw(Mac::PropertyList::Boolean);
 
-# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
+# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
 package Mac::PropertyList::false;
 use base qw(Mac::PropertyList::Boolean);
 
+
 =back
 
 =head1 SOURCE AVAILABILITY
 
-This source is part of a SourceForge project which always has the
-latest sources in SVN, as well as all of the previous releases.
+This project is in Github:
 
-	http://sourceforge.net/projects/brian-d-foy/
-
-If, for some reason, I disappear from the world, one of the other
-members of the project can shepherd this module appropriately.
+	git://github.com/briandfoy/mac-propertylist.git
 
 =head1 CREDITS
 
